//==============================================================================
//  Project:       DC-DC Wandler
//  Filename:      global_defs.h
//  Version:	   Ver 1.0
//------------------------------------------------------------------------------
//  Description:   
// 
//  Globale Definitionen
// 
//  IAR Embedded Workbench Target Descriptor MSP430 Version: 3.42A/WIN
//==============================================================================
//  History:
//  24.08.07    Ver 1.0
//==============================================================================


#ifndef _global_defs
#define _global_defs
/********************************************************************/

// DCO -Einstellung (~3MHz)
#define   DELTA             92                            // -> DC0 = 92*32768Hz = 3,014656 MHz

// *** Regler ***
#define   PWM_PERIOD        256-1                         // 256 * 1/3,014656 MHz = 84,92 s
#define   MAX_DUTY_CYCLE    ((PWM_PERIOD+1)/2)            // max. Duty-Cycle = 50%
#define   MIN_DUTY_CYCLE    1                             // min. Duty-Cycle
#define   SAMPLE_OFFSET     140                           // Abtastrate = 2 * PWM-Periodendauer, verschoben um Offset
#define   SOLL_WERT         109                           // 1,5V * 109/1023 = 160mV  -> 160mV/20R = 8mA
#define   K_PROP            32                            // Proportional Anteil
#define   K_INT             4                             // Integralanteil
#define   MAX_ERR_SUM       (32768-1) / K_INT             // integralen Fehler begrenzen, so dass
                                                          // K_Int * ErrSum < 2^15 - 1 = 32767

// *** Spannungs-berwachung ***
#define   MIN_BATT_VOLTAGE  614                           // 614/1023 * 1,5V = 0,9V (= Vcc/2)
#define   SOLAR_VOLTAGE_OFF 555                           // TODO:  (OFF > ON)
#define   SOLAR_VOLTAGE_ON  444                           // TODO:
#define   U_DIODE            85                           // Spannungsabfall ber Schottky-Diode beim Laden (0,25V / 2)
#define   MIN_CHARGE_TIME     5                           // TODO: (minimale Akkuladezeit, bevor LED wieder aktiviert werden kann)


// *** I/O-Ports ***
#define   SET_TEST          (P2OUT  |=  (0x10))           // P2.4  high
#define   RESET_TEST        (P2OUT  &= ~(0x10))           // P2.4  low
#define   TOGGLE_TEST       (P2OUT  ^=  (0x10))           // P2.4  toggeln


// *** SystemFlags ***
#define   ZERO_ERR_SUMM                 0x0001            // Flag, ob integraler Fehler = 0 gesetzt werden soll
#define   SYNC_FLAG                     0x0002            // Flag zur Synchronisierung mit PWM
#define   BATT_CHECK                    0x0004            // Flag zur berprfung der Akkuspannung
#define   SOLAR_CHECK                   0x0008            // Flag zur berprfung der Solarpannung
#define   DCO_CHECK                     0x0010            // Flag zum Einstellen der DCO-Frequenz
#define   SLEEP_FLAG                    0x0020            // Flag fr Sleep-Modus (Low Power Mode)
#define   BATT_LOW                      0x0040            // Flag fr Akkuspannung zu niedrig
#define   BATT_CHARGED                  0x0080            // Flag fr Akku ist geladen
#define   DARKNESS                      0x0100            // Flag fr Einschaltschwelle fr LED-Betrieb erreicht


// *** Statemachine ***
#define   INIT_SYSTEM                   0x00              // State: System-Initialisierung
#define   SET_DCO_FREQ                  0x02              // State: DCO-Frequenz auf ca. 3MHz einstellen
#define   PREPARE_LED_CONTROL           0x04              // State: LED-Regelung vorbereiten
#define   RE_ENTER_LED_CONTROL          0x06              // State: zur LED-Regelung zurckkehren
#define   LED_CONTROL                   0x08              // State: aktive Regelung zur LED-Konstantstromregelung
#define   CHECK_BATT_VOLTAGE_RUN        0x0A              // State: Akkuspannung berprfen (whrend LED-Betrieb)
#define   CHECK_BATT_VOLTAGE_SLEEP      0x0C              // State: Akkuspannung berprfen (whrend Sleep)
#define   CHECK_SOLAR_VOLTAGE_RUN       0x0E              // State: Solarspannung berprfen (whrend LED-Betrieb)
#define   CHECK_SOLAR_VOLTAGE_SLEEP     0x10              // State: Solarspannung berprfen (whrend Sleep)
#define   ENTER_SLEEP_MODE              0x12              // State: Vorbereitungen fr Low Power Mode (LPM3) treffen
#define   CONTINUE_SLEEP_MODE           0x14              // State: In LPM3 wechseln bzw. weiter darin verharren
#define   WAKE_UP                       0x16              // State: LPM3 verlassen und in LED-Betrieb wechseln
#define   CHECK_SYSTEM_STATE            0x18              // State: Check der Systemparameter und ggf. Wechseln des States
#define   POWER_SAVE_LPM_0              0x1A              // State: Low Power Mode 0 (whrend LED-Betrieb)

/********************************************************************/
#endif /* #ifndef __global_defs */
