//==============================================================================
//  Project:       DC-DC Wandler
//  Filename:      interrupts.c
//  Version:	   Ver 1.0
//------------------------------------------------------------------------------
//  Description:   
// 
//  Interrupt-Service-Routinen
// 
//  IAR Embedded Workbench Target Descriptor MSP430 Version: 3.42A/WIN
//==============================================================================
//  History:
//  24.08.07    Ver 1.0	
//==============================================================================


#include "interrupts.h"    
 

#pragma vector = TIMERA1_VECTOR
__interrupt void TMR_A_Interrupt (void)
{
  switch(__even_in_range(TAIV,10))
  {
    case 0x0004:                              // *** CCR2 IRQ ***
      SystemFlags ^= SYNC_FLAG;               // Sync_Flag toggeln
      break;
  }
}



#pragma vector = WDT_VECTOR
__interrupt void WDT_Interrupt (void)
{
  if(!(SystemFlags & SLEEP_FLAG))             // *** falls LED-Betrieb (Intervall = SMCLK/32768 = 10,87ms):
  {
    DCO_Timeout++;                            // Timeout-Zhler fr Set_DCO() erhhen
    WDT_Timer++;                              // WDT-IRQ Zhler erhhen
    
    if(WDT_Timer == 92)                       // nach 1s
    {
      SystemFlags |= BATT_CHECK;              // -> Batt_Check durchfhren   
      __low_power_mode_off_on_exit();
    }
    else if(WDT_Timer == 184)                 // nach 2s
    {
      SystemFlags |= SOLAR_CHECK;             // -> Solar_Check durchfhren   
      __low_power_mode_off_on_exit();
    }
    else if(WDT_Timer >= 276)                 // nach 3s
    {
      SystemFlags |= DCO_CHECK;               // -> DCO_Check durchfhren
      WDT_Timer = 0;                          // WDT-IRQ Zhler zurcksetzen
      __low_power_mode_off_on_exit();
    }
  }
  
  else                                        // *** falls Sleep-Modus (Intervall = ACLK/32768 = 1s):
  {
    WDT_Timer++;                              // WDT-IRQ Zhler erhhen

    if(WDT_Timer == 1)                        // nach 1s
    {
      SystemFlags |= BATT_CHECK;              // -> Batt_Check durchfhren  
      __low_power_mode_off_on_exit();         // Sleep-Modus (LPM3) beenden
    }
    
    if(WDT_Timer >= 2)                        // nach 2s
    {
      SystemFlags |= SOLAR_CHECK;             // -> Solar_Check durchfhren
      WDT_Timer = 0;                          // WDT-IRQ Zhler zurcksetzen    
      __low_power_mode_off_on_exit();         // Sleep-Modus (LPM3) beenden
    }
  }
}

