//==============================================================================
//  Project:       DC-DC Wandler
//  Filename:      main.c
//  Version:	   Ver 1.0
//------------------------------------------------------------------------------
//  Description:   
// 
//  Hauptprogramm
// 
//  IAR Embedded Workbench Target Descriptor MSP430 Version: 3.42A/WIN
//==============================================================================
//  History:
//  24.08.07    Ver 1.0	
//==============================================================================


#include "main.h" 


void main(void)
{
// *****************************************************************************
// *** Begin Statemachine ***

  unsigned int  NextState;                                // nchster State
  unsigned int  PrevState;                                // vorheriger State
  unsigned int  BattVoltage;                              // lokale Variable zur bergabe der Akkuspannung
                                                          // von CHECK_BATT_VOLTAGE_SLEEP an CHECK_SOLAR_VOLTAGE_SLEEP
  
  NextState = INIT_SYSTEM;                                // Start der Statemachine mit Init
  
  while(1)
  {
    switch(NextState)
    {
    // *** System-Initialisierung ***
      case INIT_SYSTEM:
        Init_System();
        NextState = SET_DCO_FREQ;
        PrevState = PREPARE_LED_CONTROL;
        break;
        
    // *** DCO-Frequenz auf etwa 3MHz einstellen ***
      case SET_DCO_FREQ:
        Set_DCO();
        NextState = PrevState;
        break;
        
    // *** Vorbereitungen fr die Regelung des LED-Stroms ***
      case PREPARE_LED_CONTROL:
        Prepare_Led_Control();
        NextState = LED_CONTROL;
        break;
        
    // *** zum Regelkreis zurckkehren ***
      case RE_ENTER_LED_CONTROL:
        Re_Enter_Led_Control();
        NextState = LED_CONTROL;
        break;
    
    // *** Regelung des LED-Stroms ***
      case LED_CONTROL:
        Led_Control();
        NextState = POWER_SAVE_LPM_0;
        break;
      
      // *** Power Save (LPM0) whrend LED-Betrieb ***
      case POWER_SAVE_LPM_0:
        
        __low_power_mode_0();                                 // enter LPM0: CPU off (PWM luft weiter!)
                                                              // Aufwecken durch WDT-IRQ jede 1s
        if(SystemFlags & BATT_CHECK)
        {
          NextState = CHECK_BATT_VOLTAGE_RUN;
          PrevState = LED_CONTROL;
        }
        
        if(SystemFlags & SOLAR_CHECK)
        {
          NextState = CHECK_SOLAR_VOLTAGE_RUN;
          PrevState = LED_CONTROL;
        }
       
        if(SystemFlags & DCO_CHECK)
        {
          NextState = SET_DCO_FREQ;
          PrevState = RE_ENTER_LED_CONTROL;
        }

        break;
        
    // *** berprfung der Akkuspannung (whrend LED-Betrieb) ***
      case CHECK_BATT_VOLTAGE_RUN:
        if(Check_Batt_Voltage_Run())
          NextState = PrevState;
        else
          NextState = ENTER_SLEEP_MODE;
        break;
        
      // *** berprfung der Akkuspannung (whrend Sleep) ***
      case CHECK_BATT_VOLTAGE_SLEEP:
        BattVoltage = Check_Batt_Voltage_Sleep();
        NextState = CHECK_SYSTEM_STATE;
        break;
        
    // *** berprfung der Solarspannung (whrend LED-Betrieb) ***
      case CHECK_SOLAR_VOLTAGE_RUN:
        if(Check_Solar_Voltage_Run())
          NextState = PrevState;
        else
          NextState = ENTER_SLEEP_MODE;
        break;
        
      // *** berprfung der Solarspannung (whrend Sleep) ***
      case CHECK_SOLAR_VOLTAGE_SLEEP:
        Check_Solar_Voltage_Sleep(BattVoltage);
        NextState = CHECK_SYSTEM_STATE;
        break;
      
      // *** Vorbereitungen fr Sleep-Modus ***
      case ENTER_SLEEP_MODE:
        Enter_Sleep_Mode();
        NextState = CONTINUE_SLEEP_MODE;
        break;
      
      // *** In den Sleep-Modus wechseln bzw. darin bleiben ***
      case CONTINUE_SLEEP_MODE:
        
        __low_power_mode_3();                                 // enter LPM3: nur ACLK ist noch aktiv (Clk fr WDT)
                                                              // Aufwecken durch WDT-IRQ jede 1s
        if(SystemFlags & BATT_CHECK)
          NextState = CHECK_BATT_VOLTAGE_SLEEP;
        
        if(SystemFlags & SOLAR_CHECK)
          NextState = CHECK_SOLAR_VOLTAGE_SLEEP;

        break;
      
      // *** berprfung, ob Sleep-Modus oder Wake-Up ***
      case CHECK_SYSTEM_STATE:
        if( (!(SystemFlags & BATT_LOW    )) &&
              (SystemFlags & BATT_CHARGED)  &&
              (SystemFlags & DARKNESS)         )
        {
          NextState = WAKE_UP;
        }
        else
        {
          NextState = CONTINUE_SLEEP_MODE;
        }
        break;
        
      // *** Sleep-Modus verlassen und in den LED-Betrieb wechseln ***
      case WAKE_UP:
        NextState = INIT_SYSTEM;
        break;

        
    }
  }
} // end main 
// *****************************************************************************




void Init_System (void)
{
  _DINT();						  // alle IRQs (global) sperren
  WDTCTL  = WDTPW + WDTHOLD;				  // WDT stopped

// *** Basic Clock Settings ***
  DCOCTL   = DCO0 + DCO1;                                 // DCO = 3, MOD = 0
  BCSCTL1  = XT2OFF + DIVA_0 + RSEL0 + RSEL1 + RSEL2;     // ACLK = LFXT1CLK(LF-Mode) = 32768Hz, RSEL = 7
  BCSCTL2  = 0;                                           // MCLK = DCOCLK, SMCLK = DCOCLK, int. DCO-Resist. 
  
// *** I/O-Ports ***
  P1OUT = 0;                                              // alle Ports Null setzen
  P2OUT = 0;
  P3OUT = 0;
  P1SEL = 0x04;                                           // P1.2 TA1-output (CCR1) fr PWM
  P2SEL = 0x06;                                           // P2.1 A1-input (LED-Strom), P2.2 A2-input (Solarspannung)
  P1DIR = 0xFF;						  // P1 output
  P2DIR	= 0xF9;						  // P2.1 u. P2.2 input, alle anderen output
  P3DIR	= 0xFF;						  // P3 output
  
// *** Setup ADC10 ***
  ADC10CTL0 &= ~ENC;                                      // ENC=0, damit andere Einstellungen gltig sind
  ADC10CTL0  =  SREF_1+ADC10SHT_2+ADC10SR;                // int.Ref = 1,5V, TSample = 16x ADC10CLK, ADC u. Ref noch aus
  ADC10CTL1  =  INCH_1+SHS_0+ADC10SSEL_0+ADC10DIV_0;      // A1, SC-bit Start, ADC10CLK=ADC10OSC, single ch. conversion
  ADC10AE    =  ADC10AE1 + ADC10AE2;                      // A1 + A2 enable
 
// *** globale Variablen ***
  DCO_Timeout  = 0;
  WDT_Timer    = 0;
  Charge_Timer = 0;
  SystemFlags  = 0;

// *** Watchdog ***
  WDTCTL  = WDTPW + WDTTMSEL + WDTCNTCL;                  // Intervall-Timer, Intervall = SMCLK/32768
  IFG1    = 0;                                            // alle NMI-Flags zurcksetzen  
  IE1     = WDTIE;                                        // WDT-IRQ aktivieren
  
// *** Setup Timer A ***
  TACTL = TASSEL_2 + TACLR;                               // SMCLK, clear
  CCR0  = PWM_PERIOD;                                     // Timer-Periode wird ber CCR0 eingestellt
  TACTL |= MC0;                                           // TMR_A up to CCR0
  
  _EINT();						  // alle IRQs (global) zulassen  
}



void Set_DCO (void)
{
  unsigned char Compare, Oldcapture = 0;

  _DINT();  
  DCO_Timeout = 0;                                        // Flag zum Abbruch des DCO-Abgleichs
  CCTL2 = CM_1 + CCIS_1 + CAP + SCS;                      // Capture on rising edge: CCI2B(ACLK), sync.
  _EINT();
  
  while (1)
  {
    CCTL2 &= ~CCIFG;                                      // Capture-Flag zurcksetzen
    while (!(CCIFG & CCTL2));                             // warten bis Capture abgeschlossen...
    {
      if(DCO_Timeout == 460)                              // falls Timeout nach ca. 5s
        WDTCTL = 0;                                       // -> Reset auslsen
    }
   
    CCTL2 &= ~CCIFG;                                      // Capture-Flag zurcksetzen
    Oldcapture = CCR2;                                    // aktueller Capture-Wert (SMCLK)
    
    while (!(CCIFG & CCTL2));                             // warten bis Capture abgeschlossen...
    {
      if(DCO_Timeout == 460)                              // falls Timeout nach ca. 5s
        WDTCTL = 0;                                       // -> Reset auslsen
    }
    
    Compare = CCR2;                                       // aktueller Capture-Wert (SMCLK)
    
    Compare = Compare - Oldcapture;                       // Differenz zum vorherigen Capture-Wert

    if (DELTA == Compare)                                 // Falls Vorgabe (SMCLK ~3MHz) erreicht:
      break;                                              // while(1) verlassen
    
    else if (DELTA < Compare)                             // DCO zu schnell
    {
      DCOCTL--;                                           // DCO heruntersetzen
      if (DCOCTL == 0xFF)                                 // DCO Unterlauf?
      {
        if (!(BCSCTL1 == (XT2OFF + DIVA_0)))              // falls mglich,
          BCSCTL1--;                                      // RSEL verkleinern
      }
    }
    
    else                                                  // DCO zu langsam
    {
      DCOCTL++;                                           // DCO heraufsetzen
      if (DCOCTL == 0x00)                                 // DCO berlauf?
        {
          if (!(BCSCTL1 == (XT2OFF + DIVA_0 + 0x07)))     // falls mglich,
            BCSCTL1++;                                    // RSEL vergrern
        }
    }
  }
  
  CCTL2        = 0;                                       // Stop Capture 2
  SystemFlags &= ~DCO_CHECK;                              // Flag zurcksetzen, Check erledigt
}



void Prepare_Led_Control (void)
{
// *** ADC10 und Ref-Spannung ein ***
  ADC10CTL0 &= ~ENC;                                      // ENC=0, damit andere Einstellungen gltig sind
  ADC10CTL0 |= (REFON + ADC10ON);                         // ADC10 u. Ref-Spannung einschalten
  ADC10CTL0 |= ENC;                                       // ADC10 re-enable

// *** Setup Reglerparameter ***
  SystemFlags &= ~SYNC_FLAG;                              // Flag zur Sync mit PWM zurcksetzen
  SystemFlags |=  ZERO_ERR_SUMM;                          // Flag: ErrSum auf '0' zurcksetzen
  
// *** Setup PWM ***
  TACTL = TASSEL_2 + TACLR;                               // SMCLK, clear
  CCTL1 = OUTMOD0 + OUTMOD1 + OUTMOD2;                    // CCR0->Set / CCR1->Reset / output an TA1
  CCR0  = PWM_PERIOD;                                     // PWM-Periode wird ber CCR0 eingestellt
  CCR1  = MIN_DUTY_CYCLE;                                 // Duty-Cycle wird ber CCR1 eingestellt; hier Startwert
                                                          
// *** Setup CCR2-IRQ ***
  CCTL2  = 0;                                             // Setzt den Zeitpunkt fr die Abtastung des Istwertes,
  CCTL2 |=  CCIE;                                         // die Berechnung der Regelung und der Ausgabe der Stellgre.
  CCR2   = SAMPLE_OFFSET;                                 // Abtastrate mit PWM synchronisieren und verschieben

  TACTL |= MC0;                                           // TMR_A up to CCR0; los geht's...
} 



void Re_Enter_Led_Control (void)
{
// *** ADC10 und Ref-Spannung ein ***
  ADC10CTL0 &= ~ENC;                                      // ENC=0, damit andere Einstellungen gltig sind
  ADC10CTL0 |= (REFON + ADC10ON);                         // ADC10 u. Ref-Spannung einschalten
  ADC10CTL0 |=  ENC;                                      // ADC10 re-enable

  // *** Setup CCR2-IRQ ***
  CCR2   = SAMPLE_OFFSET;                                 // Abtastrate mit PWM synchronisieren und verschieben
  CCTL2 |=  CCIE;                                         // und die Berechnung der Regelung
  while(!(SystemFlags & SYNC_FLAG));                      // warten auf Sync. bzgl. PWM
  while(SystemFlags & SYNC_FLAG);
}



void Led_Control (void)
{
  static signed int ErrSum;

  signed   int      Err;
  signed   int      OldErrSum;
  unsigned int      Ist_Wert;
  long              PWM;
  unsigned char     LoopCnt = 0;
  
  // ***************************************************************
  // ADC + Ref werden in 'Prepare_Led_Control'
  // bzw. in 'Check_Batt_Voltage_Run' und 'Check_Solar_Voltage_Run'
  // eingeschaltet !  
  // ***************************************************************
  
  if(SystemFlags & ZERO_ERR_SUMM)                       // falls Flag zum Zurcksetzen von ErrSumm gesetzt ist:
  {
    ErrSum = 0;                                         // integralen Fehler zu '0' setzen
    SystemFlags &= ~ZERO_ERR_SUMM;                      // Flag lschen -> 'ErrSumm = 0' erledigt
  }

  do
  {
    while(!(SystemFlags & SYNC_FLAG));                  // warten auf Sync. bzgl. PWM
    
  // *** ADC-Wandlung ***
    ADC10CTL0 |= ADC10SC;                               // Sample + conversion
    while (!(ADC10CTL0 & ADC10IFG));                    // warten, solange Conversion noch nicht beendet ist
  
    Ist_Wert = ADC10MEM;                                // Ist-Wert auslesen
   
    ADC10CTL0 &= ~ADC10IFG;                             // ADC10 IRQ-Flag zurcksetzen
  
  // *** Regelung ***
    OldErrSum = ErrSum;                                 // integralen Fehler aus letztem Durchlauf sichern
    Err       = SOLL_WERT - Ist_Wert;                   // Fehler berechnen
    ErrSum   += Err;                                    // integraler Fehler
   
    if(ErrSum > MAX_ERR_SUM)                            // integralen Fehler begrenzen, so dass
      ErrSum = MAX_ERR_SUM;                             // K_Int * ErrSum < 2^15 - 1 = 32767
    else if(ErrSum < -MAX_ERR_SUM)
      ErrSum = -MAX_ERR_SUM;
  
    PWM  = K_PROP * Err;                                // Stellgre berechnen 
    PWM += K_INT  * ErrSum;                             // Einzelberechnung, damit kein 16bit-berlauf !                    
    PWM  = PWM >> 7;                                    // >> 7 = 1/128
    
    if(PWM > MAX_DUTY_CYCLE)                            // Stellgre begrenzen:
      PWM = MAX_DUTY_CYCLE;                             // max. Duty-Cycle
    else if(PWM < MIN_DUTY_CYCLE)
      PWM = MIN_DUTY_CYCLE;                             // min. Duty-Cycle
    
    while(SystemFlags & SYNC_FLAG);                     // warten auf Sync. bzgl. PWM
    
    CCR1 = PWM;                                         // berechnete Stellgre fr PWM verwenden (CCR1 bestimmt Duty-Cycle)
    
    LoopCnt++;                                          // Schleifenzhler als Abbruchkriterium
    
  } while((ErrSum != OldErrSum) && (LoopCnt < 118));    // wiederholen, bis ErrSum konstant bleibt. Oder Abbruch nach ca. 20ms
  
  ADC10CTL0 &= ~ENC;                                    // ENC=0, damit andere Einstellungen gltig sind
  ADC10CTL0 &= ~(REFON + ADC10ON);                      // ADC10 u. Ref-Spannung ausschalten
}



char Check_Batt_Voltage_Run (void)
{
  unsigned int  ADC10RegSave;
  unsigned int  BattVoltage;
  unsigned char Return_Flag;
 
  ADC10RegSave = ADC10CTL1 & (INCH0+INCH1+INCH2+INCH3); // bisherigen ADC10-Kanal sichern
  
  ADC10CTL0 &= ~ENC;                                    // ENC=0, damit andere Einstellungen gltig sind
  ADC10CTL0 |= (REFON + ADC10ON);                       // ADC10 u. Ref-Spannung einschalten
  ADC10CTL1 &= ~(INCH0+INCH1+INCH2+INCH3);              // vorherige Kanalauswahl lschen
  ADC10CTL1 |=  INCH_15;                                // Kanal 11: Versorgungsspannung / 2
  ADC10CTL0 |=  ENC;                                    // ADC10 re-enable
  
  for(char i=0; i<15; i++)                              // Settlingtime fr ADC10-Referenzspanung
  {
    _NOP();
    _NOP();
  }
 
  while(!(SystemFlags & SYNC_FLAG));                    // warten auf Sync. bzgl. PWM   

  ADC10CTL0 |=  ADC10SC;                                // start conversion
  
  while (!(ADC10CTL0 & ADC10IFG));                      // warten, solange Conversion noch nicht beendet ist
  
  BattVoltage = ADC10MEM;                               // Ist-Wert auslesen
  
  if(BattVoltage > MIN_BATT_VOLTAGE)                    // falls Akku-Spannung ausreichend:
    Return_Flag = 1;                                    // Flag: OK
  else                                                  // falls Akku-Spannung zu niedrig:
  {
    SystemFlags |=  BATT_LOW;                           // Flag: Akkuspannung zu niedrig setzen
    SystemFlags &= ~BATT_CHARGED;                       // Flag: Akku ist geladen lschen
    Charge_Timer = 0;                                   // Akku-Ladetimer zurcksetzen
    Return_Flag  = 0;                                   // Flag: ERROR
  }
 
  ADC10CTL0 &= ~ADC10IFG;                               // ADC10 IRQ-Flag zurcksetzen
  
  ADC10CTL0 &= ~ENC;                                    // ENC=0, damit andere Einstellungen gltig sind
  ADC10CTL1 &= ~INCH_15;                                // Kanal 11 lschen
  ADC10CTL1 |= ADC10RegSave;                            // ursprnglichen Kanal wieder herstellen
  ADC10CTL0 |= ENC;                                     // ADC10 re-enable
//  ADC10CTL0 &= ~(REFON + ADC10ON);                    // ADC10 u. Ref-Spannung ausschalten
// -> wird nach 'Led_Control_Loop' ausgeschaltet
  
  while(SystemFlags & SYNC_FLAG);                       // warten auf Sync. bzgl. PWM
  
  SystemFlags &= ~BATT_CHECK;                           // Flag zurcksetzen, Check erledigt
  
  return (Return_Flag);
}



unsigned int Check_Batt_Voltage_Sleep (void)
{
  unsigned int  ADC10RegSave;
  unsigned int  BattVoltage;
  
  ADC10RegSave = ADC10CTL1 & (INCH0+INCH1+INCH2+INCH3); // bisherigen ADC10-Kanal sichern

  ADC10CTL0 &= ~ENC;                                    // ENC=0, damit andere Einstellungen gltig sind
  ADC10CTL0 |= (REFON + ADC10ON);                       // ADC10 u. Ref-Spannung einschalten
  ADC10CTL1 &= ~(INCH0+INCH1+INCH2+INCH3);              // vorherige Kanalauswahl lschen
  ADC10CTL1 |=  INCH_15;                                // Kanal 11: Versorgungsspannung / 2
  ADC10CTL0 |=  ENC;                                    // ADC10 re-enable
  
  for(char i=0; i<15; i++)                              // Settlingtime fr ADC10-Referenzspanung
  {
    _NOP();
    _NOP();
  }
  
  ADC10CTL0 |=  ADC10SC;                                // start conversion
  
  while (!(ADC10CTL0 & ADC10IFG));                      // warten, solange Conversion noch nicht beendet ist
  
  BattVoltage = ADC10MEM;                               // Ist-Wert auslesen
  
  if(BattVoltage > MIN_BATT_VOLTAGE)                    // falls Akkuspannung ausreichend ist
  {
    if(SystemFlags & BATT_CHARGED)                      // falls Akku ausreichend geladen ist
    {
      SystemFlags &= ~BATT_LOW;                         // Flag: Akkuspannung zu niedrig lschen
    }
  }
  
  else                                                  // falls Akkuspannung zu niedrig:
  {
    SystemFlags |=  BATT_LOW;                           // Flag: Akkuspannung zu niedrig setzen
    SystemFlags &= ~BATT_CHARGED;                       // Flag: Akku ist geladen lschen
    Charge_Timer = 0;                                   // Akku-Ladetimer zurcksetzen
  }
 
  ADC10CTL0 &= ~ADC10IFG;                               // ADC10 IRQ-Flag zurcksetzen
  
  ADC10CTL0 &= ~ENC;                                    // ENC=0, damit andere Einstellungen gltig sind
  ADC10CTL1 &= ~INCH_15;                                // Kanal 11 lschen
  ADC10CTL1 |= ADC10RegSave;                            // ursprnglichen Kanal wieder herstellen
  ADC10CTL0 &= ~(REFON + ADC10ON);                      // ADC10 u. Ref-Spannung ausschalten
  
  SystemFlags &= ~BATT_CHECK;                           // Flag zurcksetzen, Check erledigt
  
  return(BattVoltage);                                  // Akkuspannung zurckgeben
}



char Check_Solar_Voltage_Run (void)
{
  unsigned int  ADC10RegSave;
  unsigned int  SolarVoltage;
  unsigned char Return_Flag;
 
  ADC10RegSave = ADC10CTL1 & (INCH0+INCH1+INCH2+INCH3); // bisherigen ADC10-Kanal sichern
  
  ADC10CTL0 &= ~ENC;                                    // ENC=0, damit andere Einstellungen gltig sind
  ADC10CTL0 |= (REFON + ADC10ON);                       // ADC10 u. Ref-Spannung einschalten
  ADC10CTL1 &= ~(INCH0+INCH1+INCH2+INCH3);              // vorherige Kanalauswahl lschen
  ADC10CTL1 |=  INCH_2;                                 // Kanal 2: A2
  ADC10CTL0 |=  ENC;                                    // ADC10 re-enable
  
  for(char i=0; i<15; i++)                              // Settlingtime fr ADC10-Referenzspanung
  {
    _NOP();
    _NOP();
  }
  
  while(!(SystemFlags & SYNC_FLAG));                    // warten auf Sync. bzgl. PWM 
 
  ADC10CTL0 |=  ADC10SC;                                // start conversion
  
  while (!(ADC10CTL0 & ADC10IFG));                      // warten, solange Conversion noch nicht beendet ist
  
  SolarVoltage = ADC10MEM;                              // Ist-Wert auslesen
  
  if(SolarVoltage < SOLAR_VOLTAGE_OFF)                  // falls Solarzellenspannung kleiner als Ausschaltschwelle (Dunkelheit)
    Return_Flag = 1;                                    // Flag: OK
  
  else                                                  // falls Solarzellenspannung grer (Helligkeit):
  {
    SystemFlags &= ~BATT_LOW;                           // Flag: Akkuspannung zu niedrig zurcksetzen
    SystemFlags |=  BATT_CHARGED;                       // Flag: Akku geladen setzen
    SystemFlags &= ~DARKNESS;                           // Flag: Dunkelheit lschen
    Return_Flag = 0;                                    // Flag: ERROR
  }
 
  ADC10CTL0 &= ~ADC10IFG;                               // ADC10 IRQ-Flag zurcksetzen
  
  ADC10CTL0 &= ~ENC;                                    // ENC=0, damit andere Einstellungen gltig sind
  ADC10CTL1 &= ~INCH_2;                                 // Kanal 2 lschen
  ADC10CTL1 |= ADC10RegSave;                            // ursprnglichen Kanal wieder herstellen
  ADC10CTL0 |= ENC;                                     // ADC10 re-enable
//  ADC10CTL0 &= ~(REFON + ADC10ON);                    // ADC10 u. Ref-Spannung ausschalten
// -> wird nach 'Led_Control_Loop' ausgeschaltet

  while(SystemFlags & SYNC_FLAG);                       // warten auf Sync. bzgl. PWM 
  
  SystemFlags &= ~SOLAR_CHECK;                          // Flag zurcksetzen, Check erledigt
  
  return (Return_Flag);
}



void Check_Solar_Voltage_Sleep (unsigned int BattVoltage)
{
  unsigned int  ADC10RegSave;
  unsigned int  SolarVoltage;
 
  ADC10RegSave = ADC10CTL1 & (INCH0+INCH1+INCH2+INCH3); // bisherigen ADC10-Kanal sichern
  
  ADC10CTL0 &= ~ENC;                                    // ENC=0, damit andere Einstellungen gltig sind
  ADC10CTL0 |= (REFON + ADC10ON);                       // ADC10 u. Ref-Spannung einschalten
  ADC10CTL1 &= ~(INCH0+INCH1+INCH2+INCH3);              // vorherige Kanalauswahl lschen
  ADC10CTL1 |=  INCH_2;                                 // Kanal 2: A2
  ADC10CTL0 |=  ENC;                                    // ADC10 re-enable
  
  for(char i=0; i<15; i++)                              // Settlingtime fr ADC10-Referenzspanung
  {
    _NOP();
    _NOP();
  }
  
  ADC10CTL0 |=  ADC10SC;                                // start conversion
  
  while (!(ADC10CTL0 & ADC10IFG));                      // warten, solange Conversion noch nicht beendet ist
  
  SolarVoltage = ADC10MEM;                              // Ist-Wert auslesen
  
  if(SolarVoltage < SOLAR_VOLTAGE_ON)                   // falls Solarzellenspannung kleiner als Einschaltschwelle (Dunkelheit)
  {
    SystemFlags |= DARKNESS;                            // Flag fr Dunkelheit setzen
  }
  else                                                  // falls Solarzellenspannung grer (Helligkeit):
  {
    SystemFlags &= ~DARKNESS;                           // Flag fr Dunkelheit lschen
    
    if(SolarVoltage > BattVoltage + U_DIODE)            // falls Solarspannung > Akkuspannung + Diodensp. (Akku ldt!)
    {
      Charge_Timer++;                                   // Ladetimer inkrementieren
      
      if(Charge_Timer > MIN_CHARGE_TIME)                // falls Akku ausreichend geladen wurde
      {
        Charge_Timer = MIN_CHARGE_TIME;                 // Ladetimer festhalten -> berlauf verhindern
        SystemFlags |= BATT_CHARGED;                    // Flag: Akku geladen setzen
      }
    }
  }
 
  ADC10CTL0 &= ~ADC10IFG;                               // ADC10 IRQ-Flag zurcksetzen
  
  ADC10CTL0 &= ~ENC;                                    // ENC=0, damit andere Einstellungen gltig sind
  ADC10CTL1 &= ~INCH_2;                                 // Kanal 2 lschen
  ADC10CTL1 |= ADC10RegSave;                            // ursprnglichen Kanal wieder herstellen
  ADC10CTL0 &= ~(REFON + ADC10ON);                      // ADC10 u. Ref-Spannung ausschalten
  
  SystemFlags &= ~SOLAR_CHECK;                          // Flag zurcksetzen, Check erledigt
}



void Enter_Sleep_Mode (void)
{
  WDTCTL = WDTPW + WDTHOLD;				// WDT stopped
  
  SystemFlags |= SLEEP_FLAG;                            // Flag fr Sleep-Modus setzen
  
  TACTL = 0;                                            // Stop Timer_A 
  CCTL1 = 0;                                            // Stop PWM
  P1OUT = 0;                                            // PMW-out = 0
  
  ADC10CTL0 &= ~ENC;                                    // ENC=0, damit andere Einstellungen gltig sind
  ADC10CTL0 &= ~(REFON + ADC10ON);                      // ADC10 u. Ref-Spannung ausschalten

  WDT_Timer = 0;                                        // WDT_Timer zurcksetzen
  WDTCTL = WDTPW + WDTTMSEL + WDTCNTCL + WDTSSEL;       // Intervall, von SMCLK/32768 auf ACLK/32768 umschalten
}
